/****************************************************************************
 Module
   BallCollectionSM.c

 Revision
   1.0.1

 Description
   This is the lower level state machine under the Ball collecting State. This module implements the  
   Gen2 Events and Services Framework.

 Notes:
   Events: ES_ = System events,    EV_ = User defined events
 
 History
 When             Who               What/Why
 --------------   ----------        --------
 02/22/2012       Yushi Yang        Start 1st draft
 02/23/2012       Yushi Yang        Start 2nd draft
 02/28/2012       Ryan and Chris    3rd Draft
****************************************************************************/

/*----------------------------- Include Files -----------------------------*/
/* include header files for this state machine as well as any machines at the
   next lower level in the hierarchy that are sub-machines to this machine*/

#include "ES_Configure.h"
#include "ES_Framework.h"
#include "RobotMotionSM.h"
#include "BallCollectionSM.h"
#include "DrivePWM.h"
#include <stdio.h>

/*----------------------------- Module Defines ----------------------------*/
// define constants for this machine: Timers and Time defined in header file



/*---------------------------- Module Functions ---------------------------*/
/* prototypes for private functions for this machine, things like during
   functions, entry & exit functions.They should be functions relevant to the
   behavior of this state machine*/



/*---------------------------- Module Variables ---------------------------*/
// everybody needs a state variable, you may need others as well

static BallCollectionState CurrentState;

/*------------------------------ Module Code ------------------------------*/
/****************************************************************************
 Function
    RunBallCollectionSM

 Parameters
   unsigned char: the event to process

 Returns
   unsigned char: an event to return

 Notes
   uses nested switch/case to implement the machine.

****************************************************************************/
// make recursive call warning into info
#pragma MESSAGE INFORMATION C1855
ES_Event RunBallCollectionSM( ES_Event CurrentEvent )
{
   boolean MakeTransition = False;  // No transition made  
   BallCollectionState NextState = CurrentState;
   ES_Event ReturnEvent = CurrentEvent;     // Assume we are not consuming event
   
   switch ( CurrentState )
   {
       
       
       case RotatingToCenter:  // If current state is RotatingToCenter (Note: no during function for this state)      
             
            if ( CurrentEvent.EventType != ES_NO_EVENT )        //If an event is active
            {
					
					switch (CurrentEvent.EventType)
					{
							case ES_TIMEOUT: // If current event is timeout,then robot is done rotating, start driving forward now
								if (CurrentEvent.EventParam == BALL_COLLECTION_TIMER ) // Only respond to ball collection timer events (after rotation about 30 deg)  
								{
								   
									DriveRobot(FORWARD, 100,100);                                 // Start driving the robot forward 
								
									ES_Timer_InitTimer (BALL_COLLECTION_TIMER, DRIVE_TO_CENTER);  // Reinitialize timer to 'drive to center' time 
								                    
                           NextState = DrivingToCenter;                                  // Next State will be DrivingToCenter State 
                            
								   MakeTransition = True;                                        // Mark that we are making a transition
								}
								ReturnEvent.EventType = ES_NO_EVENT;                             // Consume this event 
							break;
                } // end of swith EventType
            }  // end of if statement
            
       break;  // break of RotatingToCenter
         
         
         
         
         
       case DrivingToCenter :   // If current state is DrivingToCenter (Note: no during function for this state)
                 
          if ( CurrentEvent.EventType != ES_NO_EVENT )        //If an event is active
            {
               switch (CurrentEvent.EventType)
               {
						  case ES_TIMEOUT: //If current event is timeout, Robot has arrived at the center with no interruptions, continue the routine
						     	if (CurrentEvent.EventParam == BALL_COLLECTION_TIMER) // Only respond to ball collection timer events (after DRIVE_TO_CENTER seconds later)
								{
																		
									DriveRobot(CLOCKWISE, 100, 100);                               // Start rotating robot clockwise
								
									ES_Timer_InitTimer (BALL_COLLECTION_TIMER, DEG90_ROTATION);   	// Reinitialize timer so that robot stops after 90deg rotation 
								                   
								   NextState = RotatingCWToForward1;                              // Next state will be RotatingCWToForward1 
								   
								   MakeTransition = True;                                         // Mark that we are taking a transition
								}
								ReturnEvent.EventType = ES_NO_EVENT;                              // Consume this event
						  break;   // break of Timeout
                
                    
                    case EV_LeftFrontBumpSensor: //If event is LeftFrontBumpSensor, either of the front bump sensor triggered
                        
                        DriveRobot(CLOCKWISE, 100, 100);                                  // Start rotating robot clockwise 
						      
							   ES_Timer_InitTimer (BALL_COLLECTION_TIMER, DEG90_ROTATION);       // Reinitialize timer so that robot stops after 90deg rotation
                     
								NextState = RotatingCWToForward1;                                 // Next state will be RotatingCWToForward1 
								
								MakeTransition = True;                                            // Mark that we are taking a transition
								
					 			ReturnEvent.EventType = ES_NO_EVENT;                              // Consume this event
			           break;   // break EV_LeftFrontBumpSensor
			           
			           
			           case EV_RightFrontBumpSensor: //If event is RightFrontBumpSensor, either of the front bump sensor triggered
                        
                        DriveRobot(COUNTER_CLOCKWISE, 100, 100);                          // Start rotating robot clockwise
						      
							   ES_Timer_InitTimer (BALL_COLLECTION_TIMER, DEG90_ROTATION);       // Reinitialize timer so that robot stops after 90deg rotation
                     
								NextState = RotatingCWToForward1;                                 // Next state will be RotatingCWToForward1
								 
								MakeTransition = True;                                            // Mark that we are taking a transition
								
					 			ReturnEvent.EventType = ES_NO_EVENT;                              // Consume this event
			           break;   // break EV_RightFrontBumpSensor 
			           
			           
               }  // end swith EventType
            }  // end if statement
       break;  // break of DrivingToCenter 
       
         
         
       
       case RotatingCWToForward1 :   // If current state is RotatingCWToForward1  (Note: no during function for this state)
                 
             if ( CurrentEvent.EventType != ES_NO_EVENT )        //If an event is active
             {
                 switch (CurrentEvent.EventType)
                  {
                     
                     case ES_TIMEOUT: //If current event is timeout, Robot has rotated and ready to move parallel to the wall
                     
                        if (CurrentEvent.EventParam == BALL_COLLECTION_TIMER) // Only respond to ball collection timer events (after DEG90_ROTATION seconds later)  
								{
																	
									DriveRobot(FORWARD, 100, 100);     	                           // Start driving forward robot 
								
									ES_Timer_InitTimer (BALL_COLLECTION_TIMER, CENTER_TO_WALL);    // Reinitialize timer so that robot stops after drives to wall
								                          
                           NextState = DrivingForwardToWall;                              // Next state is DrivingForwardToWall 
                           
                           MakeTransition = True;                                         // Mark that we are taking a transition
								}
                        ReturnEvent.EventType = ES_NO_EVENT;                              // Consume this event
                     break;
                     
                     case EV_LeftRearBumpSensor: //If event is LeftRearBumpSensor, either of the rear bump sensor triggered
                        
                        DriveRobot(FORWARD, 100, 100);                                    // Start driving forward robot 
						      
							   ES_Timer_InitTimer (BALL_COLLECTION_TIMER, CENTER_TO_WALL);       // Reinitialize timer so that robot stops after driving to wall
                     
								NextState = DrivingForwardToWall;                                 // Next state is DrivingForwardToWall 
								
								MakeTransition = True;                                            // Mark that we are taking a transition
								
					 			ReturnEvent.EventType = ES_NO_EVENT;                              // Consume this event
			            break;   // break EV_FrontTwoBumpSensor 
			           
			           
			            case EV_RightRearBumpSensor: //If event is RightRearBumpSensor, either of the rear bump sensor triggered
                        
                        DriveRobot(FORWARD, 100, 100);                                    // Start driving forward robot
						      
							   ES_Timer_InitTimer (BALL_COLLECTION_TIMER, CENTER_TO_WALL);       // Reinitialize timer so that robot stops after driving to wall
                     
								NextState = DrivingForwardToWall;                                 // Next state is DrivingForwardToWall
								 
								MakeTransition = True;                                            // Mark that we are taking a transition
								
					 			ReturnEvent.EventType = ES_NO_EVENT;                              // Consume this event
			            break;   // break EV_FrontTwoBumpSensor 
                     
                  } // end switch statement
              }// end if statement
              
       break; // break of RotatingCWToForward1 
       
       
         
         
       case DrivingForwardToWall : // If current state is DrivingForwardToWall  (Note: no during function for this state)
                
            if ( CurrentEvent.EventType != ES_NO_EVENT )        //If an event is active
            {
                 switch (CurrentEvent.EventType)
                  {
                     case ES_TIMEOUT: // If timeout occur before triggering either front sensors, (assume both front sensors are not working)
                     
                        if (CurrentEvent.EventParam == BALL_COLLECTION_TIMER) // Only respond to ball collection timer events (after CENTER_TO_WALL seconds later) 
								{
									DriveRobot(BACKWARD, 100, 100);    						             // Start backing up the robot
									
									ES_Timer_InitTimer (BALL_COLLECTION_TIMER, BACKUP_TO_GET_WALL); // Reinitialize timer so that robot stops after backup to grab the rotating wall
								                           
                           NextState = PushingBackUp;                                      // Next state is PushingBackUp
                           
                           MakeTransition = True;                                          // Mark that we are taking a transition
                           
								}
                        ReturnEvent.EventType = ES_NO_EVENT;                               // Consume this event
                     break;
                   
                     
                     case EV_RightFrontBumpSensor:   // If current event is RightFrontBumpSensor, the robot hits the playing field wall
                     
                           DriveRobot(BACKWARD, 100, 100);                                 // Start backing up the robot
                     		
                     		ES_Timer_InitTimer (BALL_COLLECTION_TIMER, BACKUP_TO_GET_WALL); // Reinitialize timer so that robot stops after backup to grab the rotating wall
                     		
                     		NextState = PushingBackUp;                                      // Next state is PushingBackUp
                     		
								   MakeTransition = True;                                          // Mark that we are taking a transition
								   
					 			   ReturnEvent.EventType = ES_NO_EVENT;                            // Consume this event
                     		
                     break;
                     
                     case EV_LeftFrontBumpSensor:   // if current event is LeftFrontBumpSensor, the robot hits the playing field wall
                     
                           DriveRobot(BACKWARD, 100, 100);                                 // Start backing up the robot
                     		
                     		ES_Timer_InitTimer (BALL_COLLECTION_TIMER, BACKUP_TO_GET_WALL); // Reinitialize timer so that robot stops after backup to grab the rotating wall
                     		
                     		NextState = PushingBackUp;                                      // Next state is PushingBackUp
                     		
								   MakeTransition = True;                                          // Mark that we are taking a transition
								   
					 			   ReturnEvent.EventType = ES_NO_EVENT;                            // Consume this event
                     		
                     break;
                                                                
                  } //  end switch statement
                  
            } // end if statement
                 
       break;  // break of DrivingForwardToWall
       
       
       
       
       case PushingBackUp: // If current state is PushingBackUp  (Note: no during function for this state)
       
            if ( CurrentEvent.EventType != ES_NO_EVENT )        //If an event is active
            {
                 switch (CurrentEvent.EventType)
                  {
                     case ES_TIMEOUT: // if current event is timeout after backup time
                        if (CurrentEvent.EventParam == BALL_COLLECTION_TIMER)   // Only respond to ball collection timer events (after BACKUP_TO_GET_WALL seconds later) 
								{
																	
									DriveRobot(CLOCKWISE, 100, 100);                               // Start rotating robot clockwise
								
									ES_Timer_InitTimer (BALL_COLLECTION_TIMER, DEG90_ROTATION);    // Reinitialize timer so that robot stops after rotating 90 deg
								                           
                           NextState = PushingRotation;                                   // Next state is PushingRotation
                            
                           MakeTransition = True;                                         // Mark that we are taking a transition
								}
                        ReturnEvent.EventType = ES_NO_EVENT;                              // Consume this event
                  
                     break;
                  } // end of switch statement
                  
            } // end of if statement
             
       break; // break of PushingBackUp
       
        
               
         
       case PushingRotation: // If current state is PushingRotation  (Note: no during function for this state)
          
            if ( CurrentEvent.EventType != ES_NO_EVENT )        //If an event is active
             {
                 switch (CurrentEvent.EventType)
                  {
                     case ES_TIMEOUT: //If event is timeout, robot has rotated to be perpendicular to the rotating wall and start to follow a curve around the perimeter
                        
                        if (CurrentEvent.EventParam == BALL_COLLECTION_TIMER)  // Only respond to ball collection timer events (after DEG90_ROTATION seconds later)  
								{
									
									DriveRobot(BACKWARD, 90, 40);                                    // Start driving the robot backward on a radius  
																	
                           ES_Timer_InitTimer (BALL_COLLECTION_TIMER, BACKUP_TO_GRAB_WALL); // Reinitialize timer so that robot stops after grabing the wall
                        
                           NextState = CollectingRotatingWall;                              // Next state is CollectingRotatingWall
                            
                           MakeTransition = True;                                           // Mark that we are taking a transition
								}
                        ReturnEvent.EventType = ES_NO_EVENT;                                // Consume this event
                     break;
                                 
                  }  // end of switch statement
                  
              } // end of if statement
              
       break;  // break of PushingRotation
       
       
       
       
      case CollectingRotatingWall:  // If current state is CollectingRotatingWall  (Note: no during function for this state)
      
             if ( CurrentEvent.EventType != ES_NO_EVENT )        // If an event is active
             {
                 switch (CurrentEvent.EventType)
                  {
                     case ES_TIMEOUT: // If current event is timeout, robot has backed up and grabbed the wall
                        if (CurrentEvent.EventParam == BALL_COLLECTION_TIMER) 
								{
									 DriveRobot(FORWARD, 100, 60);                                    // Start driving the robot on a radius, left motor speed > right motor speed
								    
								    ES_Timer_InitTimer (BALL_COLLECTION_TIMER, DRIVING_ON_RADIUS);   // Reinitialize timer so that robot stops driving on the radius
																	                              
                            NextState = DrivingOnRad1;                                       // Next state is DrivingOnRad1
                                                        
                            MakeTransition = True;                                           // Mark that we are taking a transition
								}
                        ReturnEvent.EventType = ES_NO_EVENT;                                 // Consume this event
                        
                     break;                                                                                                                                                            
                                                                                  
                   } // end switch statement
                   
              } // end if statement
              
      break;  // break of CollectingRotatingWall
      
      
         

      case DrivingOnRad1 :  // If current state is DrivingOnRad1  (Note: no during function for this state)
       
            if ( CurrentEvent.EventType != ES_NO_EVENT )        //If an event is active
             {
                  switch (CurrentEvent.EventType)
                  {
                      case EV_RightFrontBumpSensor: // If current event is RightFrontBumpSensor, robot has hit the rotating wall
                      
                           DriveRobot(BACKWARD, 100, 60);                                     // Start backing up the robot for rotation
                     	
                     		ES_Timer_InitTimer (BALL_COLLECTION_TIMER, BACKUP);                // Reinitialize timer so that timesout after backup
                     		
                     		NextState = RandomBallCollecting;                                  // Next state will be  RandomBallCollecting 
                     		
								   MakeTransition = True;                                             // Mark that we are taking a transition
								   
					 			   ReturnEvent.EventType = ES_NO_EVENT;                               // Consume this event
                     		
                     break;
                     
                     case EV_RightUpperBumpSensor:  // If current event is RightUpperBumpSensor, robot has hit the rotating wall
                     
                           DriveRobot(BACKWARD, 100, 60);                                     // Start backing up the robot for rotation
                     	
                     		ES_Timer_InitTimer (BALL_COLLECTION_TIMER, BACKUP);                // Reinitialize timer so that timesout after backup
                     		
                     		NextState = RandomBallCollecting;                                  // Next state will be  RandomBallCollecting 
                     		
								   MakeTransition = True;                                             // Mark that we are taking a transition
								   
					 			   ReturnEvent.EventType = ES_NO_EVENT;                               // Consume this event
                     		
                     break;

                     
                     case ES_TIMEOUT: //if current event is timeout, robot has done driving on a radius without triggering any of the front bump sensors
                     
                        if (CurrentEvent.EventParam == BALL_COLLECTION_TIMER)  // Only respond to ball collection timer events (after DRIVING_ON_RADIUS seconds later) 
								{
									
							    	DriveRobot(BACKWARD, 100, 60);                                     // Start backing up the robot for rotation
                     	
                     		ES_Timer_InitTimer (BALL_COLLECTION_TIMER, BACKUP);                // Reinitialize timer so that timesout after backup
                     		
                     		NextState = RandomBallCollecting;                                  // Next state will be  RandomBallCollecting 
                     		
								   MakeTransition = True;                                             // Mark that we are taking a transition
								}
                        ReturnEvent.EventType = ES_NO_EVENT;                                  // Consume this event
                        
                     break;
                       
                 } // end switch statement
	
              } // end if statement
              
       break;  // break of DrivingOnRad1
       
       


      case RandomBallCollecting:  // If current state is RandomBallCollecting  (Note: no during function for this state)
      
            if ( CurrentEvent.EventType != ES_NO_EVENT )        //If an event is active
             {
                 switch (CurrentEvent.EventType)
                  {
                        case EV_RightFrontBumpSensor: // If current event is RightFrontBumpSensor
                            
                              DriveRobot(BACKWARD, 100, 100);                                // Start backing up the robot for rotation    
                        		
                        		ES_Timer_InitTimer (BALL_COLLECTION_TIMER, BACKUP);            // Reinitialize timer so that timesout after backup
                        		
                        		ReturnEvent.EventType = ES_NO_EVENT;                           // Consume this event 
                        break;
                        

                        case EV_LeftFrontBumpSensor:  // If current event is RightFrontBumpSensor
                        
                              DriveRobot(BACKWARD, 100, 100);                                // Start backing up the robot for rotation
                        		
                        		ES_Timer_InitTimer (BALL_COLLECTION_TIMER, BACKUP);            // Reinitialize timer so that timesout after backup
                        		
                        		ReturnEvent.EventType = ES_NO_EVENT;                           // Consume this event 
                        break; 
                        
                        		
                        case ES_TIMEOUT: //If event is timeout
                        
                        	if (CurrentEvent.EventParam == BALL_COLLECTION_TIMER) // Only respond to ball collection timer events (after BACKUP seconds later) 
                        	{
                        		DriveRobot(CLOCKWISE,100,100);                                // Start rotating the robot
                        	
                        	   ES_Timer_InitTimer (BALL_RANDOM_TIMER, ROTATE);               // Initialize BALL_RANDOM_TIMER for rotation time
                        	    
                        		ReturnEvent.EventType = ES_NO_EVENT;                          // Consume this event
                        	}
                        	
                        	if (CurrentEvent.EventParam == BALL_RANDOM_TIMER)   // Only respond to ball random timer events (after ROTATE seconds later) 
                        	{
                        		DriveRobot(FORWARD,80,80);                                    // Start driving the robot forward
                        	                                                                                                                           	   
                        		ReturnEvent.EventType = ES_NO_EVENT;                          // Consume this event 
                        	}
                        break;
                  } // end switch statement
                  
              } // end if statement
              
      break; // break of RandomBallCollecting


      }// end of switch states
   
    
   if (MakeTransition == True)   //   If we are making a state transition
    {
       //  Execute exit function for current state
       CurrentEvent.EventType = ES_EXIT;
       RunBallCollectionSM(CurrentEvent);

       //Modify state variable
       CurrentState = NextState; 

       //  Execute entry function for new state
       CurrentEvent.EventType = ES_ENTRY;
       RunBallCollectionSM(CurrentEvent);
     }
     return(ReturnEvent);
}
/****************************************************************************
 Function
     StartBallCollectionSM

 Parameters
     Event_t CurrentEvent

 Returns
     None

 Description
     Does any required initialization for this state machine

****************************************************************************/
void StartBallCollectionSM ( ES_Event CurrentEvent )
{
   // local variable to get debugger to display the value of CurrentEvent
   ES_Event LocalEvent = CurrentEvent;
   
   // always start on RotatingToCenter state if enter without history
   if ( CurrentEvent.EventType != ES_ENTRY_HISTORY)
   {
        CurrentState = RotatingToCenter;
   }
   
   // call the entry function (if any) for the ENTRY_STATE
   RunBallCollectionSM(CurrentEvent);
}

/****************************************************************************
 Function
     QueryBallCollectionSM

 Parameters
     None

 Returns
     unsigned char The current state of the Template state machine

 Description
     returns the current state of the Template state machine

****************************************************************************/
BallCollectionState QueryBallCollectionSM ( void )
{
   return(CurrentState);
}

/***************************************************************************
 private functions
 ***************************************************************************/

				
