/****************************************************************************
 Module
   BallDefenseSM.c

 Revision
   1.0.1

 Description
   This is the lower level state machine under the Ball collecting State. This module implements the  
   Gen2 Events and Services Framework.

 Notes:
    States:  
    Timers: Timer2: for internal timeout; 
    Events: ES_ = System events,    EV_ = User defined events
 
 History
 When             Who               What/Why
 --------------   ----------        --------
 02/23/2012       Yushi Yang        Start 1st draft
 03/02/2012       Chris Ploch       Start Final draft 
****************************************************************************/

/*----------------------------- Include Files -----------------------------*/
/* include header files for this state machine as well as any machines at the
   next lower level in the hierarchy that are sub-machines to this machine
*/
#include "ES_Configure.h"
#include "ES_Framework.h"
#include "BallDefenseSM.h"
#include "BallCollectionSM.h"   
#include "DrivePWM.h"
#include "SPIFSM.h"
#include <stdio.h>
#include "BallDumpSM.h"

/*----------------------------- Module Defines ----------------------------*/
// define constants for this machine
// define timers (moved to header file)
//#define BALL_DEFENSE_TIMER       1

// define timeout times (needs to be set according to testing)

//#define DEFENSE_DRIVE             1000  // 1 sec

//
/*---------------------------- Module Functions ---------------------------*/
/* prototypes for private functions for this machine, things like during
   functions, entry & exit functions.They should be functions relevant to the
   behavior of this state machine

 Entry Function: Start Rotating CCW;
 Exit Function: Stop Motor;
 During function: no lower level state machine needs to be executed
 */


/*---------------------------- Module Variables ---------------------------*/
// everybody needs a state variable, you may need others as well
static BallDefenseState CurrentState;

/*------------------------------ Module Code ------------------------------*/
/****************************************************************************
 Function
    RunBallDefenseSM

 Parameters
   unsigned char: the event to process

 Returns
   unsigned char: an event to return

 Description
   add your description here
 Notes
   uses nested switch/case to implement the machine.

****************************************************************************/
// make recursive call warning into info
#pragma MESSAGE INFORMATION C1855
ES_Event RunBallDefenseSM( ES_Event CurrentEvent )
{
   boolean MakeTransition = False;/* are we making a state transition? */
   BallDefenseState NextState = CurrentState;
   ES_Event ReturnEvent = CurrentEvent; // assume we are not consuming event

   switch ( CurrentState )
   {
       
      case InitialDefensePosition:    // If this state is InitialDefensePosition (Note: no during function for this state) 
       
            if ( CurrentEvent.EventType != ES_NO_EVENT )  //If an event is active
            {
					switch (CurrentEvent.EventType)
   					{
   							case EV_WallApproachCW:                                     // If event is WallApproachCW
   								
   								DriveRobot(FORWARD, 80, 50);                             // Drive the robot forward and slightly CW (i.e. Left wheel > Right wheel)
   								 
   								ES_Timer_InitTimer (BALL_DEFENSE_TIMER, DEFENSE_DRIVE);  // Initialize timer so that times out when drive to edge
   								
   								NextState = DrivingForwardCW;                            // Next State will be DrivingForwardCW State
   								
   								MakeTransition = True;                                   // Mark that we are taking a transition
   								
   								ReturnEvent.EventType = ES_NO_EVENT;                     // Consume this event 
   							break;

   							case EV_WallApproachCCW:                                    // If event is WallApproachCCW
   							
   								DriveRobot(FORWARD, 50, 80);                             // Drive the robot forward and slightly CCW (i.e. Left wheel < Right wheel)
   								 
   								ES_Timer_InitTimer (BALL_DEFENSE_TIMER, DEFENSE_DRIVE);  // Initialize timer so that times out when drive to edge
   								
   								NextState = DrivingForwardCCW;                           // Next State will be DrivingForwardCCW State
   								
   								MakeTransition = True;                                   // Mark that we are taking a transition
   								
   								ReturnEvent.EventType = ES_NO_EVENT;                     // Consume this event 
      						break;     					   
					  
                    } // end switch statement   
               } // end if statement
               
      break;  // break of InitialDefensePosition
      
      
      case DrivingForwardCCW:     // If this state is DrivingForwardCW,  (Note: no during function for this state)

            if ( CurrentEvent.EventType != ES_NO_EVENT )        // If an event is active
            {
					switch (CurrentEvent.EventType)
					{
							case ES_TIMEOUT:                                               // If event is timeout
								if (CurrentEvent.EventParam == BALL_DEFENSE_TIMER)          // Only respond to  BALL_DEFENSE_TIMER 
								{
									
									DriveRobot(STOP,0,0);                                    // Stop robot
									
	              				NextState = GuardingLeftSide;                            // Next State will be GuardingLeftSide State
	              				
									MakeTransition = True;                                   // Mark that we are taking a transition
									
									ReturnEvent.EventType = ES_NO_EVENT;                     // Consume this event 
								}
							break;
                } // end switch statement
            } // end if statement
            
       break;  // break of DrivingForwardCCW
       
       
       
       case DrivingForwardCW:    // If this state is DrivingForwardCW,   (Note: no during function for this state)
    
            if ( CurrentEvent.EventType != ES_NO_EVENT )        //If an event is active
            {
					switch (CurrentEvent.EventType)
					{
							case ES_TIMEOUT:                                               //If event is timeout
								if (CurrentEvent.EventParam == BALL_DEFENSE_TIMER)          // Only respond to  BALL_DEFENSE_TIMER  
								{
									
									DriveRobot(STOP,0,0);                                    // Stop robot
									
	              				NextState = GuardingRightSide;                           // Next State will be GuardingRightSide State
	              				
									MakeTransition = True;                                   // Mark that we are taking a transition
									
									ReturnEvent.EventType = ES_NO_EVENT;                     // Consume this event 
								}
							break;
                } // end switch statement
            } // end if statement
            
       break;  // break of DrivingForwardCW  
       
       
       
       case GuardingLeftSide:    // If this state is GuardingLeftSide  (Note: no during function for this state)

            if ( CurrentEvent.EventType != ES_NO_EVENT )        //If an event is active
            {
					switch (CurrentEvent.EventType)
					{
							case EV_WallLeaveCW:                                         // If event is WallLeaveCW
								
								DriveRobot(BACKWARD, 50, 80);                             // Drive the robot reverse and slightly CW (i.e. Left wheel < Right wheel)
								
								ES_Timer_InitTimer (BALL_DEFENSE_TIMER, DEFENSE_DRIVE);   // Initialize timer so that times out when drive to initial defense position
								 
								NextState = DrivingReverseCW;                             // Next State will be DrivingReverseCW State
								
								MakeTransition = True;                                    // Mark that we are taking a transition
								
								ReturnEvent.EventType = ES_NO_EVENT;                      // Consume this event 
							break;
                } // end switch statement
            } // end if statement
            
      break;  // break of GuardingLeftSide
      
      

	    case GuardingRightSide:    // If this state is GuardingRightSide (Note: no during function for this state)

            if ( CurrentEvent.EventType != ES_NO_EVENT )        //If an event is active
            {
					switch (CurrentEvent.EventType)
					{
							case EV_WallLeaveCCW:                                       // If event is WallLeaveCCW
								
								DriveRobot(BACKWARD, 80, 50);                            // Drive the robot reverse and slightly CCW (i.e. Left wheel > Right wheel)
								 
								ES_Timer_InitTimer (BALL_DEFENSE_TIMER, DEFENSE_DRIVE);  // Initialize timer so that timesout when drive to initial defense position
								
								NextState = DrivingReverseCCW;                           // Next State will be DrivingReverseCCW State
								
								MakeTransition = True;                                   // Mark that we are taking a transition
								
								ReturnEvent.EventType = ES_NO_EVENT;                     // Consume this event
							break;
               } // end switch statement
            } // end if statement
            
      break;  // break of GuardingRightSide



       
       case DrivingReverseCCW:    // If this state is DrivingReverseCCW, (Note: no during function for this state)

            if ( CurrentEvent.EventType != ES_NO_EVENT )        //If an event is active
            {
					switch (CurrentEvent.EventType)
					{
							case EV_RearTwoBumpSensor:                                  //If event is RearTwoBumpSensor, then robot has arrived at its initial defense position
								
								DriveRobot(STOP, 0, 0);                                  // Stop the robot
								 
								NextState = InitialDefensePosition;                      //Next State will be InitialDefensePosition State
								
								MakeTransition = True;                                   // Mark that we are taking a transition
								
								ReturnEvent.EventType = ES_NO_EVENT;                     // Consume this event 
							break;   
												
							

							case ES_TIMEOUT:                                            // If event is timeout
								if (CurrentEvent.EventParam == BALL_DEFENSE_TIMER)       // Only respond to BALL_DEFENSE_TIMER  
								{
									 DriveRobot(STOP, 0, 0);                              // Stop the robot
									 
								    NextState = InitialDefensePosition;                  // Next State will be InitialDefensePosition State
								    
								    MakeTransition = True;                               // Mark that we are taking a transition
								    
								    ReturnEvent.EventType = ES_NO_EVENT;                 // Consume this event
								}
							break;
                } // end switch statement
            } // enf if statement
            
      break;  // break of DrivingReverseCCW

       
      case DrivingReverseCW:    // If this state is DrivingReverseCW, (Note: no during function for this state)

            if ( CurrentEvent.EventType != ES_NO_EVENT )        //If an event is active
            {
					switch (CurrentEvent.EventType)
					{
							case EV_RearTwoBumpSensor:                                  // If event is RearTwoBumpSensor
							
								DriveRobot(STOP, 0, 0);                                  // Stop the robot
								 
								NextState = InitialDefensePosition;                      // Next State will be InitialDefensePosition State
								
								MakeTransition = True;                                   // Mark that we are taking a transition
								
								ReturnEvent.EventType = ES_NO_EVENT;                     // Consume this event (
								
							break; 

							case ES_TIMEOUT:                                            // If event is timeout
								if (CurrentEvent.EventParam == BALL_DEFENSE_TIMER)       // Only respond to BALL_DEFENSE_TIMER    
								{
									 DriveRobot(STOP, 0, 0);                              // Stop the robot
									 
								    NextState = InitialDefensePosition;                  // Next State will be InitialDefensePosition State
								    
								    MakeTransition = True;                               // Mark that we are taking a transition
								    
								    ReturnEvent.EventType = ES_NO_EVENT;                 // Consume this event
								}
							break;
               } // end switch statement
            } // end if statement
            
      break;  // break of DrivingReverseCW
       
       
  }// End of switch states
   
    //   If we are making a state transition
   if (MakeTransition == True)
    {
       //   Execute exit function for current state
       CurrentEvent.EventType = ES_EXIT;
       RunBallDefenseSM(CurrentEvent);

       CurrentState = NextState; //Modify state variable

       //   Execute entry function for new state
       CurrentEvent.EventType = ES_ENTRY;
       RunBallDefenseSM(CurrentEvent);
     }
     return(ReturnEvent);
}
/****************************************************************************
 Function
     StartBallDefenseSM

 Parameters
     Event_t CurrentEvent

 Returns
     None

 Description
     Does any required initialization for this state machine

****************************************************************************/
void StartBallDefenseSM ( ES_Event CurrentEvent )
{
   // local variable to get debugger to display the value of CurrentEvent
   ES_Event LocalEvent = CurrentEvent;
   // always start on InitialDefensePosition state
   CurrentState = InitialDefensePosition;
}

/****************************************************************************
 Function
     QueryBallDefenseSM

 Parameters
     None

 Returns
     unsigned char The current state of the Template state machine

 Description
     returns the current state of the Template state machine

****************************************************************************/
BallDefenseState QueryBallDefenseSM ( void )
{
   return(CurrentState);
}

