/****************************************************************************
 Module
   DrivePWM.c Module

 Revision
   1.0.1

 Description
   This is the module for sending PWM commands to drive motors.
   This module sets the direction of the motor and the polarity of the PWM pin
 
 Notes:
   Pin Assignments:
   Left Motor: PWM = U0; Break = PT4; DIR = PT5; ThF = PT6;   
   Right Motor: PWM = U1; Break = PU4; DIR = PU3; ThF = PU2; 
 
 
 History
 When             Who               What/Why
 --------------   ----------        --------
 
****************************************************************************/

/*----------------------------- Include Files -----------------------------*/
/* include header files for this state machine as well as any machines at the
   next lower level in the hierarchy that are sub-machines to this machine*/
    
#include "s12evec.h"      
#include "DrivePWM.h"

/*------------------------------ Module Code ------------------------------*/
/****************************************************************************
 Function
    void DrivePWM_Init(void);

 Parameters
   None

 Returns
   None

 Notes
   Initialize PWM ports for driving and PWM.
   U0: Control left motor
   U1: Control right motor
     

****************************************************************************/

void DrivePWM_Init(void)
{
   // Set Port U0 and U1 as output ports
 
   DDRU |= (BIT0HI | BIT1HI);
 
   //Enable PWM on U0 and U1 on E128
   PWME |= (_S12_PWME0 | _S12_PWME1);

   // First clear the Prescale clk
   PWMPRCLK &= ~( _S12_PCKA0 | _S12_PCKA1 | _S12_PCKA2 ); 

  
   //Set Presale as /4 (Note: Change the prescale number in #define Prescale_CLKA) 
   PWMPRCLK |= Prescale_CLKA; 
 
   // Set polarity of Port U0 and U1
   PWMPOL |= (_S12_PPOL0 | _S12_PPOL1);
 
   //Set to use the Scaled clock
   PWMCLK |= (_S12_PCLK0 | _S12_PCLK1);
 
 
   //Set Scale for 500Hz 
   PWMSCLA = PWMScale_Target_500Hz;

   //No center align enabled
   PWMCAE =  0x00 ;                                 

   //Set Period to 100
   PWMPER0 = LMotor_Period;
   PWMPER1 = RMotor_Period;
 

   //Enable PWM for Port U0 and U1
   MODRR |= (_S12_MODRR0 | _S12_MODRR1);
 
   //printf("Finish PWM Initialization \n\r");
    
   return; 
}

/****************************************************************************
 Function
    DrivePWM_SetDutyCycle(unsigned char Motor, unsigned char DutyCycle)

 Parameters
    unsigned char Motor (0 = Left Motor; 1 = Right Motor), unsigned char DutyCycle 

 Returns
    None

 Notes
    Sets the duty cycle for the drive motors

****************************************************************************/

void DrivePWM_SetDutyCycle(unsigned char Motor, unsigned char DutyCycle)
{
    if (Motor == L_MOTOR)
    {
        //PWMDTY0 = 0;
       PWMDTY0 = DutyCycle; // Set Left motor PWM duty cycle according to user input DutyCycle
       
       
    } else if (Motor == R_MOTOR)
    {
        //PWMDTY1 = 0;
       PWMDTY1 = DutyCycle; // Set Right motor PWM duty cycle according to user input DutyCycle
       
    } else
    {
      puts("Error in SetDutyCycle");
    }

    return; 
}


/****************************************************************************
 Function
    DriveRobot(unsigned char Movement, unsigned char DC_Left, unsigned char DC_Right);

 Parameters
   Robot Movement, Duty Cycle of left wheel, Duty Cycle of right wheel
   
 Returns
   None

 Notes
   Sets the direction and duty cycle for the drive motors.
   
   Pin Assignments:
   Left Motor: PWM = U0; Break = PT4; DIR = PT5; ThF = PT6;   
   Right Motor: PWM = U1; Break = PU4; DIR = PU3; ThF = PU2; 

****************************************************************************/

                                                                        
void DriveRobot(unsigned char Movement, unsigned char DC_Left, unsigned char DC_Right)
{

   switch (Movement)
   {
           
      case  FORWARD:
      {
                 
         PTT &= BIT5LO;  // Set Left Motor Dir pin low (LMD18200)                            
         PTU |= BIT3HI;  // Set Right Motor Dir pin high (LMD18200)                        
         PTT &= BIT4LO;  // Set Left Motor BRK pin low (LMD18200)                         
         PTU &= BIT4LO;  // Set Right Motor BRK pin low (LMD18200)
         
         DrivePWM_SetDutyCycle(L_MOTOR, DC_Left);       // Set Left Motor Duty cycle
         DrivePWM_SetDutyCycle(R_MOTOR, DC_Right);      // Set Right Motor Duty cycle 
         
         printf("Current Movement: Forward \n\r");
             
      }
      break;
      
      
      case  BACKWARD:
      {
         // Both Motors: PWM = H; DIR = L; BRK = L;
         
         PTT |= BIT5HI;  // Set Left Motor Dir pin high (LMD18200)
         PTU &= BIT3LO;  // Set Right Motor Dir pin low (LMD18200)
         PTT &= BIT4LO;  // Set Left Motor BRK pin low (LMD18200)
         PTU &= BIT4LO;  // Set Right Motor BRK pin low (LMD18200) 
         
         DrivePWM_SetDutyCycle(L_MOTOR, DC_Left);       // Set Left Motor Duty cycle
         DrivePWM_SetDutyCycle(R_MOTOR, DC_Right);      // Set Right Motor Duty cycle  
         
         printf("Current Movement: Backward \n\r");
      }
      break;
      
      case  CLOCKWISE:
      {

         
         PTT &= BIT5LO;  // Set Left Motor Dir pin low (LMD18200)
         PTU &= BIT3LO;  // Set Right Motor Dir pin low (LMD18200)
         PTT &= BIT4LO;  // Set Left Motor BRK pin low (LMD18200)
         PTU &= BIT4LO;  // Set Right Motor BRK pin low (LMD18200) 
         
         DrivePWM_SetDutyCycle(L_MOTOR, DC_Left);       // Set Left Motor Duty cycle
         DrivePWM_SetDutyCycle(R_MOTOR, DC_Right);      // Set Right Motor Duty cycle  
         
         printf("Current Movement: CW \n\r");
                       
      }
      break;
      
      case  COUNTER_CLOCKWISE:
      {
         
         PTT |= BIT5HI;  // Set Left Motor Dir pin high (LMD18200)
         PTU |= BIT3HI;  // Set Right Motor Dir pin high (LMD18200)
         PTT &= BIT4LO;  // Set Left Motor BRK pin low (LMD18200)
         PTU &= BIT4LO;  // Set Right Motor BRK pin low (LMD18200) 
         
         DrivePWM_SetDutyCycle(L_MOTOR, DC_Left);       // Set Left Motor Duty cycle
         DrivePWM_SetDutyCycle(R_MOTOR, DC_Right);      // Set Right Motor Duty cycle   
         printf("Current Movement: CCW \n\r"); 
      }
      break;
      
      case STOP:
      {
         // Both Motors: PWM = H; DIR = L; BRK = H;
         PTU |= BIT0HI;  // Set Left Motor PWM pin high (LMD18200)
         PTU |= BIT1HI;  // Set Right Motor PWM pin high (LMD18200) 
         PTT &= BIT5LO;  // Set Left Motor Dir pin low (LMD18200)
         PTU &= BIT3LO;  // Set Right Motor Dir pin low (LMD18200)
         PTT |= BIT4HI;  // Set Left Motor BRK pin high (LMD18200)
         PTU |= BIT4HI;  // Set Right Motor BRK pin high (LMD18200)
         
         DrivePWM_SetDutyCycle(L_MOTOR, DC_Left);       // Set Left Motor Duty cycle
         DrivePWM_SetDutyCycle(R_MOTOR, DC_Right);      // Set Right Motor Duty cycle 
         
         printf("Current Movement: Stop \n\r");     
      }
      break;
   }  
     
}












