/****************************************************************************
 Module
   EventCheckers.c

 Revision
   1.0.1

 Description
   This module lists the user defined event checkers

 Notes:
   Events: ES_ = System events,    EV_ = User defined events
 
 History
 When             Who               What/Why
 --------------   ----------        --------

****************************************************************************/
/*----------------------------- Include Files -----------------------------*/
/* include header files for this state machine as well as any machines at the
   next lower level in the hierarchy that are sub-machines to this machine*/
#include "ES_Configure.h"
#include "ES_General.h"
#include "ES_Events.h"
#include "ES_PostList.h"
#include "EventCheckers.h"


#include <stdio.h>
#include <hidef.h>         
#include "ME218_E128.h"
#include <ads12.h>
#include <string.h>
#include "SPIFSM.h"
#include "BallDumpSM.h"
#include "RobotMotionSM.h"
#include "ES_ServiceHeaders.h"


/*------------------------------ Module Code ------------------------------*/
/****************************************************************************
 Function
    CheckFrontTapeSensor(void)

 Parameters
   None

 Returns
   boolean, true if front tape sensor is detected


****************************************************************************/

boolean CheckFrontTapeSensor(void)
{
   static unsigned char LastFrontTapeSensorState;
   unsigned char CurrentFrontTapeSensorState;
   boolean ReturnVal = False;
   
   CurrentFrontTapeSensorState = PTIAD & BIT0HI;   // Read current input state from pin AD0
   
   //If the current pin reading is high and different from last time, then event detected
   if ((CurrentFrontTapeSensorState != LastFrontTapeSensorState) && (CurrentFrontTapeSensorState == BIT0HI))
   {
      // Post event:  EV_FrontTapeSensor
      ES_Event ThisEvent;
      ThisEvent.EventType = EV_FrontTapeSensor;
      ThisEvent.EventParam = 1;
      PostRobotMotionSM(ThisEvent);
            
      ReturnVal = True;      
   }
   
  LastFrontTapeSensorState = CurrentFrontTapeSensorState; // Assign last pin state to current pin state
   
   return ReturnVal;

}


/****************************************************************************
 Function
    CheckBackTapeSensor(void)

 Parameters
   None

 Returns
   boolean, true if back tape sensor is detected


****************************************************************************/
boolean CheckBackTapeSensor(void)
{
   static unsigned char LastBackTapeSensorState;
   unsigned char CurrentBackTapeSensorState;
   boolean ReturnVal = False;
   
   CurrentBackTapeSensorState = PTIAD & BIT1HI;      // Read current input state from pin AD1
   
   //If the current pin reading is high and different from last time, then event detected
   if ((CurrentBackTapeSensorState != LastBackTapeSensorState) && (CurrentBackTapeSensorState == BIT1HI))
   {
      // Post event: EV_BackTapeSensor; 
      ES_Event ThisEvent;
      ThisEvent.EventType = EV_BackTapeSensor;
      ThisEvent.EventParam = 1;
      PostRobotMotionSM(ThisEvent);
      
      ReturnVal = True;
      
   }
  
  LastBackTapeSensorState = CurrentBackTapeSensorState;   // Assign last pin state to current pin state
   
  return ReturnVal;

}


/****************************************************************************
 Function
    CheckLeftFrontBumpSensor(void)

 Parameters
   None

 Returns
   boolean, true if Left Front Bump sensor is triggered


****************************************************************************/
boolean CheckLeftFrontBumpSensor(void)
{
   static unsigned char LastLFBumpSensorState;
   unsigned char CurrentLFBumpSensorState;
   boolean ReturnVal = False;
   
   CurrentLFBumpSensorState = PTP & BIT3HI;       // Read current input state from pin PP3
   
   //If the current pin reading is high and different from last time, then event detected
   if ((CurrentLFBumpSensorState != LastLFBumpSensorState) && (CurrentLFBumpSensorState == BIT3HI))
   {
      // Post event: EV_LeftFrontBumpSensor;
      ES_Event ThisEvent;
      ThisEvent.EventType = EV_LeftFrontBumpSensor;
      ThisEvent.EventParam = 1;
      PostRobotMotionSM(ThisEvent);
      
      ReturnVal = True;
      
   }
   LastLFBumpSensorState = CurrentLFBumpSensorState;  // Assign last pin state to current pin state
   
   return ReturnVal;

}



/****************************************************************************
 Function
    CheckRightFrontBumpSensor(void)

 Parameters
   None

 Returns
   boolean, true if Right Front Bump sensor is triggered


****************************************************************************/
boolean CheckRightFrontBumpSensor(void)
{
   static unsigned char LastRFBumpSensorState;
   unsigned char CurrentRFBumpSensorState;
   boolean ReturnVal = False;
   
   CurrentRFBumpSensorState = PTP & BIT1HI;     // Read current input state from pin PP1
  
   //If the current pin reading is high and different from last time, then event detected
   if ((CurrentRFBumpSensorState != LastRFBumpSensorState) && (CurrentRFBumpSensorState == BIT1HI))
   {
      // Post event: EV_LeftFrontBumpSensor;
      ES_Event ThisEvent;
      ThisEvent.EventType = EV_RightFrontBumpSensor;
      ThisEvent.EventParam = 1;
      PostRobotMotionSM(ThisEvent);

      ReturnVal = True;  
   }
   LastRFBumpSensorState = CurrentRFBumpSensorState;  // Assign last pin state to current pin state
   
   return ReturnVal;

}


/****************************************************************************
 Function
    CheckBothFrontBumpSensors(void)

 Parameters
   None

 Returns
   boolean, true if Both Front Bump sensor is triggered


****************************************************************************/
boolean CheckBothFrontBumpSensors(void)
{
   unsigned char CurrentLFBumpSensorState;
   unsigned char CurrentRFBumpSensorState;
   static unsigned char bumpers;
   boolean ReturnVal = False;
   
   CurrentLFBumpSensorState = PTP & BIT3HI;        // Read current left bumper input state from pin PP3                                             
   CurrentRFBumpSensorState = PTP & BIT1HI;        // Read current right bumper input state from pin PP1
   
   //If both pin inputs are high and different from last time, then event detected
   if ((CurrentLFBumpSensorState == BIT3HI) && (CurrentRFBumpSensorState == BIT1HI) && (bumpers != 0x0A))
   {
      // Post event: EV_FrontTwoBumpSensor
      ES_Event ThisEvent;
      ThisEvent.EventType = EV_FrontTwoBumpSensor;
      ThisEvent.EventParam = 1;
      PostRobotMotionSM(ThisEvent);

      ReturnVal = True;
      
   } 
   
   bumpers =  CurrentLFBumpSensorState | CurrentRFBumpSensorState;  // Assign bumpers to be current left front sensor state or current right front sensor state 
   
   return ReturnVal;

}



/****************************************************************************
 Function
    CheckLeftUpperBumpSensor(void)

 Parameters
   None

 Returns
   boolean, true if Left Upper Bump sensor is triggered


****************************************************************************/
boolean CheckLeftUpperBumpSensor(void)
{
   static unsigned char LastLUBumpSensorState;
   unsigned char CurrentLUBumpSensorState;
   boolean ReturnVal = False;
   
   CurrentLUBumpSensorState = PTIAD & BIT4HI;      // Read current left upper bumper input state from pin AD4 
   
   //If the current pin reading is high and different from last time, then event detected
   if ((CurrentLUBumpSensorState != LastLUBumpSensorState) && (CurrentLUBumpSensorState == BIT4HI))
   {
      // Post event:  EV_LeftUpperBumpSensor
      ES_Event ThisEvent;
      ThisEvent.EventType = EV_LeftUpperBumpSensor;
      ThisEvent.EventParam = 1;
      PostRobotMotionSM(ThisEvent);
      
      ReturnVal = True;   
   }
   LastLUBumpSensorState = CurrentLUBumpSensorState;   // Assign last pin state to current pin state
   
   return ReturnVal;

}




/****************************************************************************
 Function
    CheckRightUpperBumpSensor(void)

 Parameters
   None

 Returns
   boolean, true if Right Upper Bump sensor is triggered


****************************************************************************/
boolean CheckRightUpperBumpSensor(void)
{
   static unsigned char LastRUBumpSensorState;
   unsigned char CurrentRUBumpSensorState;
   boolean ReturnVal = False;
   
   CurrentRUBumpSensorState = PTIAD & BIT5HI;         // Read current right upper bumper input state from pin AD5
   
   //If the current pin reading is high and different from last time, then event detected
   if ((CurrentRUBumpSensorState != LastRUBumpSensorState) && (CurrentRUBumpSensorState == BIT5HI))
   {
      // Post event:  EV_RightUpperBumpSensor
      ES_Event ThisEvent;
      ThisEvent.EventType = EV_RightUpperBumpSensor;
      ThisEvent.EventParam = 1;
      PostRobotMotionSM(ThisEvent);
      
      ReturnVal = True;
      
   }
   LastRUBumpSensorState = CurrentRUBumpSensorState;    // Assign last pin state to current pin state
   
   return ReturnVal;

}



/****************************************************************************
 Function
    CheckBothUpperBumpSensor(void)

 Parameters
   None

 Returns
   boolean, true if Both Upper Bump sensors are triggered


****************************************************************************/ 

boolean CheckBothUpperBumpSensors(void)
{
   unsigned char CurrentLUBumpSensorState;
   unsigned char CurrentRUBumpSensorState;
   static unsigned char bumpers;                                                  
   boolean ReturnVal = False;
   
   CurrentLUBumpSensorState = PTIAD & BIT4HI;         // Read current left upper bumper input state from pin AD4                                            
   CurrentRUBumpSensorState = PTIAD & BIT5HI;         // Read current right upper bumper input state from pin AD5
   
   //If the current pin readings are high and different from last time, then event detected
   if ((CurrentLUBumpSensorState == BIT4HI) && (CurrentRUBumpSensorState == BIT5HI) && (bumpers != 0x30))
   {
      // Post event: EV_UpperTwoBumpSensor;
      ES_Event ThisEvent;
      ThisEvent.EventType = EV_UpperTwoBumpSensor;
      ThisEvent.EventParam = 1;
      PostRobotMotionSM(ThisEvent);

      ReturnVal = True;
      
   } 
   
   bumpers =  CurrentLUBumpSensorState | CurrentRUBumpSensorState;    // Assign bumpers to be current left upper sensor state or current right upper sensor state 
   
   return ReturnVal;

}


/****************************************************************************
 Function
    CheckLeftRearBumpSensor(void)

 Parameters
   None

 Returns
   boolean, true if Left Rear Bump sensor is triggered


****************************************************************************/

boolean CheckLeftRearBumpSensor(void)
{
   static unsigned char LastLRBumpSensorState;
   unsigned char CurrentLRBumpSensorState;
   boolean ReturnVal = False;
   
   CurrentLRBumpSensorState = PTIAD & BIT6HI;     // Read current left rear bumper input state from pin AD6
  
   //If the current pin reading is high and different from last time, then event detected
   if ((CurrentLRBumpSensorState != LastLRBumpSensorState) && (CurrentLRBumpSensorState == BIT6HI))
   {
      // Post event: EV_LeftRearBumpSensor;
      ES_Event ThisEvent;
      ThisEvent.EventType = EV_LeftRearBumpSensor;
      ThisEvent.EventParam = 1;
      PostRobotMotionSM(ThisEvent);

      ReturnVal = True;
      
   }
   LastLRBumpSensorState = CurrentLRBumpSensorState;   // Assign last pin state to current pin state 
   
   return ReturnVal;

}



/****************************************************************************
 Function
    CheckRightRearBumpSensor(void)

 Parameters
   None

 Returns
   boolean, true if Right Rear Bump sensor is triggered


****************************************************************************/
boolean CheckRightRearBumpSensor(void)
{
   static unsigned char LastRRBumpSensorState;
   unsigned char CurrentRRBumpSensorState;
   boolean ReturnVal = False;
   
   CurrentRRBumpSensorState = PTIAD & BIT7HI;       // Read current right rear bumper input state from pin AD7
  
   //If the current pin reading is high and different from last time, then event detected
   if ((CurrentRRBumpSensorState != LastRRBumpSensorState) && (CurrentRRBumpSensorState == BIT7HI))
   {
      
      // Post event: EV_RightRearBumpSensor;
      ES_Event ThisEvent;
      ThisEvent.EventType = EV_RightRearBumpSensor;
      ThisEvent.EventParam = 1;
      PostRobotMotionSM(ThisEvent);

      ReturnVal = True;
      
   }
   
   LastRRBumpSensorState = CurrentRRBumpSensorState;    // Assign last pin state to current pin state 
   
   return ReturnVal;

}




/****************************************************************************
 Function
    CheckBothRearBumpSensors(void)

 Parameters
   None

 Returns
   boolean, true if Both Rear Bump sensors are triggered


****************************************************************************/
boolean CheckBothRearBumpSensors(void)
{
   unsigned char CurrentLRBumpSensorState;
   unsigned char CurrentRRBumpSensorState;
   static unsigned char bumpers;
   boolean ReturnVal = False;
   
   CurrentLRBumpSensorState = PTIAD & BIT6HI;       // Read current left rear bumper input state from pin AD6                                               
   CurrentRRBumpSensorState = PTIAD & BIT7HI;       // Read current right rear bumper input state from pin AD7
   
   //If the current pin readings are high and different from last time, then event detected
   if ((CurrentLRBumpSensorState == BIT6HI) && (CurrentRRBumpSensorState == BIT7HI) && (bumpers != 0xC0))
   {
      // Post event:  EV_RearTwoBumpSensor
      ES_Event ThisEvent;
      ThisEvent.EventType = EV_RearTwoBumpSensor;
      ThisEvent.EventParam = 1;
      PostRobotMotionSM(ThisEvent);

      ReturnVal = True;
      
   } 
   
   bumpers =  CurrentLRBumpSensorState | CurrentRRBumpSensorState;   // Assign bumpers to be current left rear sensor state or current right rear sensor state 
   
   return ReturnVal;

}




/****************************************************************************
 Function
    CheckWallDefenseApproach(void)

 Parameters
   None

 Returns
   Post events based on the current wall angle and defense bin


****************************************************************************/
boolean CheckWallDefenseApproach(void)
{
   unsigned char wall_status;
   unsigned char bin;
   boolean ReturnVal = False;
   unsigned char team;
   wall_status = QueryStatus(AR_WALL_ANGLE)*2;          // Query wall statue from SPI
   bin = QueryDefenseBin();                             // Query defense bin number
   team = QueryTeam();                                  // Query team
   
 if(team == REDTEAM)    //if we are on the red team
 {
                            
   switch(bin)          // switch based on our defense bin 
   {
      case 1:  // If defense bin is 1
            
         // if wall angle is greater than 329 degrees and smaller than 31 degrees
         if ((wall_status > ANGLE_H) || (wall_status < ANGLE_A))
         {
            // Post event: EV_WallApproachCCW
            ES_Event ThisEvent;
            ThisEvent.EventType = EV_WallApproachCCW;
            PostRobotMotionSM(ThisEvent); 
            
            ReturnVal = True;
            
         }
         
         // if wall angle is greater than 54 degrees and smaller than 121 degrees
         if ((wall_status > ANGLE_B) && (wall_status < ANGLE_C))
         {
            // Post event: EV_WallApproachCW
            ES_Event ThisEvent;
            ThisEvent.EventType = EV_WallApproachCW;
            PostRobotMotionSM(ThisEvent); 

            ReturnVal = True;
            
         }
      break;
      
      
      case 2:  // if defense bin is 2
      
         // if wall angle is greater than 54 degrees and smaller than 121 degrees
         if ((wall_status > ANGLE_B) && (wall_status < ANGLE_C))
         {
            // Post event: EV_WallApproachCCW
            ES_Event ThisEvent;
            ThisEvent.EventType = EV_WallApproachCCW;
            PostRobotMotionSM(ThisEvent); 

            ReturnVal = True;
            
         }
         
         // if wall angle is greater than 151 degrees and smaller than 215 degrees
         if ((wall_status > ANGLE_D) && (wall_status < ANGLE_E))
         {
            // Post event: EV_WallApproachCW
            ES_Event ThisEvent;
            ThisEvent.EventType = EV_WallApproachCW;
            PostRobotMotionSM(ThisEvent); 

            ReturnVal = True;
            
         }
      break;
      
      
         
      case 3: // if defense bin is 3
      
         // if wall angle is greater than 151 degrees and smaller than 215 degrees
         if ((wall_status > ANGLE_D) && (wall_status < ANGLE_E))
         {
            // Post event: EV_WallApproachCCW
            ES_Event ThisEvent;
            ThisEvent.EventType = EV_WallApproachCCW;
            PostRobotMotionSM(ThisEvent); 

            ReturnVal = True;
            
         }
         
         // if wall angle is greater than 240 degrees and smaller than 303 degrees
         if ((wall_status > ANGLE_F) && (wall_status < ANGLE_G))
         {
            // Post event: EV_WallApproachCW
            ES_Event ThisEvent;
            ThisEvent.EventType = EV_WallApproachCW;
            PostRobotMotionSM(ThisEvent); 

            ReturnVal = True;
            
         }
      break;
      
      
         
      case 4: // if defense bin is 4
      
         // if wall angle is greater than 240 degrees and smaller than 303 degrees
         if ((wall_status > ANGLE_F) && (wall_status < ANGLE_G))
         {
            // Post event: EV_WallApproachCCW
            ES_Event ThisEvent;
            ThisEvent.EventType = EV_WallApproachCCW;
            PostRobotMotionSM(ThisEvent); 

            ReturnVal = True;
            
         }
         
         // if wall angle is greater than 329 degrees and smaller than 31 degrees
         if ((wall_status > ANGLE_H) || (wall_status < ANGLE_A))
         {
            // Post event: EV_WallApproachCW
            ES_Event ThisEvent;
            ThisEvent.EventType = EV_WallApproachCW;
            PostRobotMotionSM(ThisEvent); 

            ReturnVal = True;
            
         }
      break;
   }

 } else if(team == BLUETEAM)   // else if we are on the blue team
 {
      
   switch(bin)   // switch cases based on current defense bin
   {
      case 1:  // if defense bin is 1
      
         // if wall angle is greater than 151 degrees and smaller than 215 degrees
         if ((wall_status > ANGLE_D) && (wall_status < ANGLE_E))
         {
            // Post event: EV_WallApproachCCW
            ES_Event ThisEvent;
            ThisEvent.EventType = EV_WallApproachCCW;
            PostRobotMotionSM(ThisEvent); 

            ReturnVal = True;
            
         }
         
         // if wall angle is greater than 240 degrees and smaller than 303 degrees
         if ((wall_status > ANGLE_F) && (wall_status < ANGLE_G))
         {
            // Post event: EV_WallApproachCW
            ES_Event ThisEvent;
            ThisEvent.EventType = EV_WallApproachCW;
            PostRobotMotionSM(ThisEvent); 

            ReturnVal = True;
            
         }
      break;
      
      
      
      case 2: // if defense bin is 2
      
         // if wall angle is greater than 240 degrees and smaller than 303 degrees
         if ((wall_status > ANGLE_F) && (wall_status < ANGLE_G))
         {
            // Post event: EV_WallApproachCCW
            ES_Event ThisEvent;
            ThisEvent.EventType = EV_WallApproachCCW;
            PostRobotMotionSM(ThisEvent); 

            ReturnVal = True;
            
         }
         
         // if wall angle is greater than 329 degrees and smaller than 31 degrees
         if ((wall_status > ANGLE_H) || (wall_status < ANGLE_A))
         {
            // Post event: EV_WallApproachCCW
            ES_Event ThisEvent;
            ThisEvent.EventType = EV_WallApproachCW;
            PostRobotMotionSM(ThisEvent); 

            ReturnVal = True;
            
         }
      break;
      
      
         
      case 3:   // if defense bin is 3
      
         // if wall angle is greater than 329 degrees and smaller than 31 degrees
         if ((wall_status > ANGLE_H) || (wall_status < ANGLE_A))
         {
            // Post event: EV_WallApproachCCW
            ES_Event ThisEvent;
            ThisEvent.EventType = EV_WallApproachCCW;
            PostRobotMotionSM(ThisEvent); 

            ReturnVal = True;
            
         }
         
         // if wall angle is greater than 54 degrees and smaller than 121 degrees
         if ((wall_status > ANGLE_B) && (wall_status < ANGLE_C))
         {
            // Post event: EV_WallApproachCW
            ES_Event ThisEvent;
            ThisEvent.EventType = EV_WallApproachCW;
            PostRobotMotionSM(ThisEvent); 

            ReturnVal = True;
            
         }
      break;
      
      
         
      case 4: // if defense bin is 4      
         
         // if wall angle is greater than 54 degrees and smaller than 121 degrees
         if ((wall_status > ANGLE_B) && (wall_status < ANGLE_C))
         {
            // Post event: EV_WallApproachCCW
            ES_Event ThisEvent;
            ThisEvent.EventType = EV_WallApproachCCW;
            PostRobotMotionSM(ThisEvent); 

            ReturnVal = True;
            
         }
         
         // if wall angle is greater than 151 degrees and smaller than 215 degrees
         if ((wall_status > ANGLE_D) && (wall_status < ANGLE_E))
         {
            // Post event: EV_WallApproachCW
            ES_Event ThisEvent;
            ThisEvent.EventType = EV_WallApproachCW;
            PostRobotMotionSM(ThisEvent); 

            ReturnVal = True;
            
         }
      break;
   }

 }
   return(ReturnVal);
}
    



