/****************************************************************************
 Module
   RobotMotionSM.c

 Revision
   1.0.1

 Description
   This module is the master state machine for controlling the robot motion. 
   This module implements the Gen2 Events and Services Framework.

 Notes:
    States:  
    Timers: Timer0: 2 min timeout; Timer1: for GameOverSoon timeout   
    Events: ES_ = System events,    EV_ = User defined events
 
 History
 When             Who               What/Why
 --------------   ----------        --------
 02/22/2012       Yushi Yang        Start 1st draft
 02/23/2012       Yushi Yang        Finished 1st draft
****************************************************************************/

/*----------------------------- Include Files -----------------------------*/
/* include header files for this state machine as well as any machines at the
   next lower level in the hierarchy that are sub-machines to this machine
*/
#include "ES_Configure.h"
#include "ES_Framework.h"


#include "RobotMotionSM.h"
#include "DrivePWM.h"
#include "SPIFSM.h"
#include "BallCollectionSM.h"
#include "BallDumpSM.h"
#include "BallDefenseSM.h"

#include <stdio.h>

/*----------------------------- Module Defines ----------------------------*/
// define constants for this machine (#defines in header file)


/*---------------------------- Module Functions ---------------------------*/
/* prototypes for private functions for this machine, things like during
   functions, entry & exit functions.They should be functions relevant to the
   behavior of this state machine

*/

static ES_Event DuringBallCollectingState(ES_Event Event);
static ES_Event DuringBallDumpingState(ES_Event Event);
static ES_Event DuringDefendingState(ES_Event Event);

void TeamRED(void);
void TeamBLUE(void);


/*---------------------------- Module Variables ---------------------------*/
// everybody needs a state variable, you may need others as well
static uint8_t MyPriority;
static RobotMotionState CurrentState;
static unsigned char team;

/*------------------------------ Module Code ------------------------------*/
/****************************************************************************
 Function
     InitRobotMotionSM

 Parameters
     uint8_t : the priorty of this service

 Returns
     boolean, False if error in initialization, True otherwise

 Description
     Saves away the priority,  and starts the top level state machine
 Notes
****************************************************************************/
boolean InitRobotMotionSM ( uint8_t Priority )          
{
  ES_Event ThisEvent;

  // Save our priority
  MyPriority = Priority;  

  ThisEvent.EventType = ES_ENTRY;
  
  // Start the Master State machine
  StartRobotMotionSM( ThisEvent );

  return True;
}


/****************************************************************************
 Function
     PostRobotMotionSM

 Parameters
     ES_Event ThisEvent , the event to post to the queue

 Returns
     boolean False if the post operation failed, True otherwise

 Description
     Posts an event to this state machine's queue
 Notes

****************************************************************************/
boolean PostRobotMotionSM( ES_Event ThisEvent )
{
  return ES_PostToService( MyPriority, ThisEvent);
}


/****************************************************************************
 Function
    RunRobotMotionSM

 Parameters
   ES_Event: the event to process

 Returns
   ES_Event: an event to return

 Description
   the run function for the top level state machine of the HierMuWave
 Notes
   uses nested switch/case to implement the machine.

****************************************************************************/

// make recursive call warning into info
#pragma MESSAGE INFORMATION C1855
ES_Event RunRobotMotionSM( ES_Event CurrentEvent )
{
   boolean MakeTransition = False;              // Assume no transitions are made 
   RobotMotionState NextState = CurrentState;
   ES_Event EntryEventKind = { ES_ENTRY, 0 };   // Default to normal entry to new state
   ES_Event ReturnEvent =  { ES_NO_EVENT, 0};   // Assume we are not consuming event
   
    switch ( CurrentState )
	{   
		case WaitingState :  // If current state is WaitingState (Note; no during functions for this state)                

		   if ( CurrentEvent.EventType != ES_NO_EVENT )   // If an event is active
         {
               switch (CurrentEvent.EventType)
               {
                  case EV_20msBeaconDetected :                                               // If Beacon 1 is detected we are team BLUE
                        TeamBLUE();
                        ReturnEvent.EventType = ES_NO_EVENT;                                 // Consume this event
   				         
                  break;
                  
                  case EV_16msBeaconDetected :                                               // If Beacon 3 is detected we are team RED 
                        TeamRED();
   				         ReturnEvent.EventType = ES_NO_EVENT;                                 // Consume this event
                  break; 
                  
                  case EV_GameOn :                                                           // If event is GameOn from SPI
                   
   				         ES_Timer_InitTimer (GAME_ON_TIMER, GAME_TIME);                       // Initialize GAME_ON_TIMER for total game on time (2 min)
                        
                        ES_Timer_InitTimer (ROBOT_MOTION_TIMER, TOTAL_BALL_COLLECTION_TIME); // Initialize ROBOT_MOTION_TIMER for total ball collecting time
                        
   				         DriveRobot(COUNTER_CLOCKWISE, 50, 50);   				                  // Start rotating the robot clockwise
   				         
                        ES_Timer_InitTimer (BALL_COLLECTION_TIMER, ROTATION_TO_CENTER_1ST);  // Initialize BALL_COLLECTION_TIMER to ROTATION_TO_CENTER_1ST ms
                           	                	
                        NextState = BallCollectingState;                                     // Next State is BallCollectingState 
                        
   				         MakeTransition = True;                                               // Mark that we are taking a transition
   				         
   				         ReturnEvent.EventType = ES_NO_EVENT;                                 // Consume this event
                  break;    
                              
               } // end switch 
                
          }// end if     
        
      break; // break WaitingState;



	   case BallCollectingState : // If current state is BallCollectingState
		   
		  CurrentEvent = DuringBallCollectingState(CurrentEvent); // Call the During function to run RunBallCollectionSM
     	  
     	  if ( CurrentEvent.EventType != ES_NO_EVENT )        //If an event is active
          {
			  switch (CurrentEvent.EventType)
				{
					case ES_TIMEOUT : 
					    
						if (CurrentEvent.EventParam == ROBOT_MOTION_TIMER)                         // if event is timeout from ROBOT_MOTION after TOTAL_BALL_COLLECTION_TIME  
					    {
						      DriveRobot(CLOCKWISE, 70,70);                                        // Start rotating the robot clockwise to look for beacon
						      
						      NextState = BallDumpingState;                                        // Next state is BallDumpingState
						      
						      MakeTransition = True;                                               // Mark that we are taking a transition
						}			
					break;
							
				} // end switch
				
		   } // end if
		   
       break; // break BallCollectingState;



	   case BallDumpingState :   // If current state is BallDumpingState
		   	    
    	  CurrentEvent = DuringBallDumpingState(CurrentEvent);    // Call the During function to run RunBallDumpSM

     	  if ( CurrentEvent.EventType != ES_NO_EVENT )      // If an event is active
          {
			  switch (CurrentEvent.EventType)
				{
					case EV_FinishDumping :                                                       // If event is FinishDumping
						
							NextState = DefendingState;                                             // Next State is DefendingState
							
					   	MakeTransition = True;                                                  // Mark that we are taking a transition
					break;
					
					case ES_TIMEOUT :                                                             // If event is Gameover TimeOut
					     	if (CurrentEvent.EventParam == GAME_ON_TIMER)                             
					      {
						      DriveRobot(STOP,0,0);                                                // Stop the robot
						      
						      PTS |= BIT2HI;                                                       // Turn on both LEDs
                        PTS |= BIT3HI;
						       
						      NextState = WaitingState;                                            // Next State is WaitingState
						      
						      MakeTransition = True;                                               // Mark that we are taking a transition
					      }
					break;
				}
		   }
       break; // break of BallDumpingState;


      case DefendingState :   // If current state is BallDumpingState
		   
		  CurrentEvent = DuringDefendingState(CurrentEvent);  // Call During function ro run RunBallDefenseSM
		  
     	  if ( CurrentEvent.EventType != ES_NO_EVENT )        //If an event is active
          {
			    switch (CurrentEvent.EventType)
				 {
				   	case ES_TIMEOUT :                                                          // If event is Gameover TimeOut
				   	
					     	if (CurrentEvent.EventParam == GAME_ON_TIMER)                           
					      {
						      DriveRobot(STOP,0,0);                                                // Stop the robot
						      
						      PTS |= BIT2HI;                                                       // Turn on both LEDs
                        PTS |= BIT3HI;
						       
						      NextState = WaitingState;                                            // Next State is WaitingState
						      
						      MakeTransition = True;                                               // Mark that we are taking a transition
					      }
					   break;
				 }
		   }
       break; // break DefendingState;

	} // end of switch states
   
   //   If we are making a state transition
    if (MakeTransition == True)
    {
       // Execute exit function for current state
       CurrentEvent.EventType = ES_EXIT;
       RunRobotMotionSM(CurrentEvent);

       //Modify state variable
       CurrentState = NextState; 

       // Execute entry function for new state
       RunRobotMotionSM(EntryEventKind);
     }
     return(ReturnEvent);

}

/****************************************************************************
 Function
     StartRobotMotionSM

 Parameters
     ES_Event CurrentEvent

 Returns
     nothing

 Description
     Does any required initialization for this state machine

****************************************************************************/
void StartRobotMotionSM ( ES_Event CurrentEvent )     
{
   // local variable to get debugger to display the value of CurrentEvent
   volatile ES_Event LocalEvent = CurrentEvent;

   // always start at WaitingState
   CurrentState = WaitingState;
   
   // call the entry function (if any) for the ENTRY_STATE
   RunRobotMotionSM(LocalEvent);
   return;
}



/****************************************************************************
 Function
     QueryRobotMotionSM

 Parameters
     None

 Returns
     RobotMotionState The current state of the RobotMotion state machine

 Description
     returns the current state of the RobotMotion state machine
 
****************************************************************************/
RobotMotionState QueryRobotMotionSM ( void )
{
   return(CurrentState);
}


/***************************************************************************
 private functions
 ***************************************************************************/
 
/****************************************************************************
 Function
     DuringBallCollectingState

 Parameters
     ES_Event Event

 Returns
     ES_Event

 Description
     During function for BallCollectingState

 Notes

****************************************************************************/
static ES_Event DuringBallCollectingState( ES_Event Event )
{
    volatile ES_Event NewEvent = Event;
	// Process ES_ENTRY & ES_EXIT events
    if ( (Event.EventType == ES_ENTRY) || (Event.EventType == ES_ENTRY_HISTORY) )
    {
         // BallCollectingState entry actions: none
         // Start BallCollectionSM
         StartBallCollectionSM(Event);
                 
    }else if ( Event.EventType == ES_EXIT)
    {
         // On exit, give the lower levels a chance to clean up first
         // BallCollectingState exit actions: none
         NewEvent = RunBallCollectionSM(Event);
		   
        
    }else
    {
        // Do the 'during' function for this state
        // Run RunBallCollectionSM
        NewEvent = RunBallCollectionSM(Event);
    }
    return(NewEvent);
}


/****************************************************************************
 Function
     DuringBallDumpingState

 Parameters
     ES_Event Event

 Returns
     ES_Event

 Description
     During function for BallCollectingState

 Notes

****************************************************************************/
static ES_Event DuringBallDumpingState(ES_Event Event)
{
    volatile ES_Event NewEvent = Event;
    
    // Process ES_ENTRY & ES_EXIT events
    if ( (Event.EventType == ES_ENTRY) || (Event.EventType == ES_ENTRY_HISTORY) )
    {
         // DuringBallDumpingState entry actions: None
		   // Start BallDumpSM
		   StartBallDumpSM(Event);
         
        
    }else if ( Event.EventType == ES_EXIT)
    {
         // On exit, give the lower levels a chance to clean up first
         // BallDumpingState exit actions: none
        NewEvent = RunBallDumpSM(Event);
	     
        
    }else
    {
        // Do the 'during' function for this state
        // run RunBallCollectionSM
        NewEvent = RunBallDumpSM(Event);
    }
    return(NewEvent);
}



/****************************************************************************
 Function
     DuringDefendingState

 Parameters
     ES_Event Event

 Returns
     ES_Event

 Description
     During function for BallCollectingState

 Notes

****************************************************************************/
static ES_Event DuringDefendingState(ES_Event Event)
{
    volatile ES_Event NewEvent = Event;
    
	 // Process ES_ENTRY & ES_EXIT events
    if ( (Event.EventType == ES_ENTRY) || (Event.EventType == ES_ENTRY_HISTORY) )
    {
       // DuringDefendingState entry actions: None
       // Start BallDefenseSM 
		 StartBallDefenseSM(Event);
        
    }else if ( Event.EventType == ES_EXIT)
    {
        // On exit, give the lower levels a chance to clean up first
        NewEvent = RunBallDefenseSM(Event);
		  
		  // DuringDefendingState exit actions: Stop Motors
        DriveRobot(STOP, 0, 0);
        
    }else
    {
        // Do the 'during' function for this state
        // run RunBallCollectionSM
        NewEvent = RunBallDefenseSM(Event);
    }
    return(NewEvent);
}





/****************************************************************************
 Function
     TeamRED(void)

 Parameters
     None

 Returns
     None

 Description
     Assign team at the beginning

 Notes

****************************************************************************/
void TeamRED(void)
{
   PTS |= BIT2HI;        // Turn on red LED
   PTS &= BIT3LO;        // Turn off blue LED
     
   team = REDTEAM;       // Assign team to REDTEAM
}



/****************************************************************************
 Function
     TeamBLUE(void)

 Parameters
     None

 Returns
     None

 Description
     Assign team at the beginning

 Notes

****************************************************************************/

void TeamBLUE(void)
{   
   PTS &= BIT2LO;          // Turn off red LED
   PTS |= BIT3HI;          // Turn on blue LED
    
   team = BLUETEAM;        // Assign team to BLUETEAM
   
}


/****************************************************************************
 Function
     QueryTeam(void)

 Parameters
     None

 Returns
     unsigned char current team

****************************************************************************/
unsigned char QueryTeam( void )
{
   return(team);
}